@@ -18,17 +18,20 @@ from __future__ import annotations
 
 import asyncio
 from collections.abc import Callable
+from datetime import timedelta
 import logging
 import random
 import struct
+import time
 from typing import TYPE_CHECKING, Any, TypeVar, cast
 import uuid
 
-from bleak import BleakClient
+from bleak.backends.device import BLEDevice
 from bleak.exc import BleakError
 
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
+    AccessoryNotFoundError,
     AuthenticationError,
     InvalidError,
     UnknownError,
@@ -38,6 +41,7 @@ from aiohomekit.model import (
     AccessoriesState,
     Accessory,
     CharacteristicsTypes,
+    Transport,
 )
 from aiohomekit.model.characteristics import CharacteristicPermissions
 from aiohomekit.model.services import ServicesTypes
@@ -48,7 +52,8 @@ from aiohomekit.protocol.tlv import TLV
 from aiohomekit.utils import async_create_task
 from aiohomekit.uuid import normalize_uuid
 
-from ..abstract import AbstractPairing
+from ..abstract import AbstractPairing, AbstractPairingData
+from .bleak import BLEAK_EXCEPTIONS, AIOHomeKitBleakClient
 from .client import (
     ble_request,
     drive_pairing_state_machine,
@@ -66,9 +71,12 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
+DISCOVER_TIMEOUT = 30
+AVAILABILITY_INTERVAL = 1800  # 30 minutes
+NEVER_TIME = -AVAILABILITY_INTERVAL
 SERVICE_INSTANCE_ID = "E604E95D-A759-4817-87D3-AA005083A0D1"
 MAX_CONNECT_ATTEMPTS = 3
-SUBSCRIPTION_RESTORE_DELAY = 3
+SUBSCRIPTION_RESTORE_DELAY = 1
 SKIP_SYNC_SERVICES = {
     ServicesTypes.THREAD_TRANSPORT,
     ServicesTypes.PAIRING,
@@ -93,34 +101,39 @@ class BlePairing(AbstractPairing):
     This represents a paired HomeKit IP accessory.
     """
 
-    pairing_id: str
-    description: HomeKitAdvertisement | None
+    description: HomeKitAdvertisement
     controller: BleController
 
-    _encryption_key: EncryptionKey | None = None
-    _decryption_key: DecryptionKey | None = None
-
-    client: BleakClient | None = None
-
-    # Used to keep track of which characteristics we already started
-    # notifications for
-    _notifications: set[int]
-
-    def __init__(self, controller: BleController, pairing_data):
+    def __init__(
+        self,
+        controller: BleController,
+        pairing_data: AbstractPairingData,
+        device: BLEDevice | None = None,
+        client: AIOHomeKitBleakClient | None = None,
+        description: HomeKitAdvertisement | None = None,
+    ) -> None:
         super().__init__(controller)
 
         self.id = pairing_data["AccessoryPairingID"]
-
+        self.device = device
+        self.client = client
         self.pairing_data = pairing_data
+        self.description = description
+        self.controller = controller
+        self._last_seen = time.monotonic() if description else NEVER_TIME
 
-        self._session_id = None
+        # Encryption
         self._derive = None
+        self._session_id = None
+        self._encryption_key: EncryptionKey | None = None
+        self._decryption_key: DecryptionKey | None = None
 
-        self._notifications = set()
+        # Used to keep track of which characteristics we already started
+        # notifications for
+        self._notifications: set[int] = set()
 
         # Only allow one attempt to aquire the connection at a time
         self._connection_lock = asyncio.Lock()
-
         # We don't want to read/write from characteristics in parallel
         # * If 2 coroutines read from the same char at the same time there
         #   would be a race error - a read result could be overwritten by another.
@@ -128,40 +141,78 @@ class BlePairing(AbstractPairing):
         #   a read/write need to be atomic otherwise we end up having
         #   to guess what encryption counter to use for the decrypt
         self._ble_request_lock = asyncio.Lock()
-
         # Only allow a single operation at at time
         self._operation_lock = asyncio.Lock()
-
         # Only allow a single attempt to sync config at a time
         self._config_lock = asyncio.Lock()
+        # Only subscribe to characteristics one at a time
+        self._subscription_lock = asyncio.Lock()
 
         self._restore_subscriptions_timer: asyncio.TimerHandle | None = None
 
-    def get_address(self) -> str:
-        """Return the most current address for the device."""
-        return self.address
-
     @property
-    def address(self):
+    def address(self) -> str:
+        """Return the address of the device."""
         return (
-            self.description.address
-            if self.description
+            self.device.address
+            if self.device
             else self.pairing_data["AccessoryAddress"]
         )
 
     @property
     def name(self):
+        """Return the name of the pairing."""
         if self.description:
             return f"{self.description.name} ({self.address})"
         return self.address
 
     @property
     def is_connected(self) -> bool:
-        return self.client and self.client.is_connected and self._encryption_key
+        return bool(self.client and self.client.is_connected and self._encryption_key)
+
+    @property
+    def is_available(self) -> bool:
+        """Returns true if the device is currently available."""
+        return self._is_available_at_time(time.monotonic())
+
+    @property
+    def poll_interval(self) -> timedelta:
+        """Returns how often the device should be polled."""
+        if any(a.needs_polling for a in self.accessories):
+            # Currently only used for devices that have energy data
+            return timedelta(minutes=5)
+        return timedelta(hours=24)
+
+    def _is_available_at_time(self, monotonic: float) -> bool:
+        """Check if we are considered available at the given time."""
+        return self.is_connected or monotonic - self._last_seen < AVAILABILITY_INTERVAL
+
+    @property
+    def transport(self) -> Transport:
+        """The transport used for the connection."""
+        return Transport.BLE
+
+    def _async_ble_device_update(self, device: BLEDevice) -> None:
+        """Update the BLE device."""
+        if self.device and device.address != self.device.address:
+            logger.debug(
+                "BLE address changed from %s to %s; closing connection",
+                self.device.address,
+                device.address,
+            )
+            async_create_task(self.close())
+        self.device = device
 
     def _async_description_update(self, description: HomeKitAdvertisement | None):
+        """Update the description of the accessory."""
+        now = time.monotonic()
+        was_available = self._is_available_at_time(now)
+        self._last_seen = now
+        if not was_available:
+            self._callback_availability_changed(True)
         if self.description != description:
             logger.debug("%s: Description updated: %s", self.address, description)
+
         repopulate_accessories = False
         if description and self.description:
             if description.config_num > self.description.config_num:
@@ -180,22 +231,14 @@ class BlePairing(AbstractPairing):
                 )
                 async_create_task(self._async_process_disconnected_events())
 
-            if description.address != self.description.address:
-                logger.debug(
-                    "BLE address changed from %s to %s; closing connection",
-                    self.description.address,
-                    description.address,
-                )
-                async_create_task(self.close())
-
         super()._async_description_update(description)
         if repopulate_accessories:
-            async_create_task(self._populate_accessories_and_characteristics())
+            async_create_task(self._async_process_config_changed())
 
     async def _async_request(
         self, opcode: OpCode, iid: int, data: bytes | None = None
     ) -> bytes:
-        char = self._accessories.aid(1).characteristics.iid(iid)
+        char = self.accessories.aid(1).characteristics.iid(iid)
         endpoint = get_characteristic(self.client, char.service.type, char.type)
         async with self._ble_request_lock:
             if not self.client or not self.client.is_connected:
@@ -216,7 +259,7 @@ class BlePairing(AbstractPairing):
                 )
             return result_data
 
-    def _async_disconnected(self, client: BleakClient) -> None:
+    def _async_disconnected(self, client: AIOHomeKitBleakClient) -> None:
         """Called when bleak disconnects from the accessory closed the connection."""
         logger.debug("%s: Session closed callback", self.name)
         self._async_reset_connection_state()
@@ -233,35 +276,37 @@ class BlePairing(AbstractPairing):
     async def _ensure_connected(self):
         if self.client and self.client.is_connected:
             return
-
         async with self._connection_lock:
+            # Check again while holding the lock
+            if self.client and self.client.is_connected:
+                return
+            if not self.device:
+                self.device = await self.controller.async_get_ble_device(
+                    self.address, DISCOVER_TIMEOUT
+                )
+            if not self.device:
+                raise AccessoryNotFoundError(
+                    f"{self.name}: Could not find {self.address}"
+                )
             self.client = await establish_connection(
-                self.name, self.get_address, self._async_disconnected
+                self.client,
+                self.device,
+                self.name,
+                self._async_disconnected,
             )
             logger.debug(
                 "%s: Connected, processing subscriptions: %s",
                 self.name,
                 self.subscriptions,
             )
-            # The MTU will always be 23 if we do not fetch it
-            #
-            #  Currently doesn't work, and we need to store it forever since
-            #  it will not change
-            #
-            # if (
-            #    self.client.__class__.__name__ == "BleakClientBlueZDBus"
-            #    and not self.client._mtu_size
-            # ):
-            #    try:
-            #        await self.client._acquire_mtu()
-            #    except (RuntimeError, StopIteration) as ex:
-            #        logger.debug("%s: Failed to acquire MTU: %s", ex, address)
+            # Only start active subscriptions if we stay connected for more
+            # than subscription delay seconds.
+            self._restore_subscriptions_timer = asyncio.get_event_loop().call_later(
+                SUBSCRIPTION_RESTORE_DELAY, self._restore_subscriptions
+            )
 
     async def _async_start_notify(self, iid: int) -> None:
-        if not self._accessories:
-            return
-
-        char = self._accessories.aid(1).characteristics.iid(iid)
+        char = self.accessories.aid(1).characteristics.iid(iid)
 
         # Find the GATT Characteristic object for this iid
         service = self.client.services.get_service(char.service.type)
@@ -287,33 +332,30 @@ class BlePairing(AbstractPairing):
                         listener(results)
 
         def _callback(id, data) -> None:
+            logger.debug("%s: Received event for iid=%s: %s", self.name, iid, data)
+            if data != b"":
+                # We should only poll on empty messages, otherwise we may poll
+                # the device every second on DBUS systems.
+                return
             if max_callback_enforcer.locked():
                 # Already one being read now, and one pending
                 return
             async_create_task(_async_callback())
 
         logger.debug("%s: Subscribing to iid: %s", self.name, iid)
-        await self.client.start_notify(endpoint, _callback)
+        try:
+            await self.client.start_notify(endpoint, _callback)
+        except ValueError:
+            await self.client.stop_notify(endpoint)
+            await self.client.start_notify(endpoint, _callback)
         self._notifications.add(iid)
 
     async def _async_pair_verify(self):
-        # If resume fails, we are allowed to try again
-        # without the previous derive and session_id but
-        # that is not yet implemented
-        try:
-            session_id, derive = await drive_pairing_state_machine(
-                self.client,
-                CharacteristicsTypes.PAIR_VERIFY,
-                get_session_keys(self.pairing_data, self._session_id, self._derive),
-            )
-        # FIXME: this should not be a broad except handler
-        except Exception:  # pylint: disable=broad-except
-            logger.debug("%s: Failed to resume, doing full", self.name)
-            session_id, derive = await drive_pairing_state_machine(
-                self.client,
-                CharacteristicsTypes.PAIR_VERIFY,
-                get_session_keys(self.pairing_data),
-            )
+        session_id, derive = await drive_pairing_state_machine(
+            self.client,
+            CharacteristicsTypes.PAIR_VERIFY,
+            get_session_keys(self.pairing_data, self._session_id, self._derive),
+        )
         self._encryption_key = EncryptionKey(
             derive(b"Control-Salt", b"Control-Write-Encryption-Key")
         )
@@ -325,11 +367,34 @@ class BlePairing(AbstractPairing):
         self._session_id = session_id
         self._derive = derive
 
+    async def _async_process_config_changed(self) -> None:
+        """Handle config changed seen from the advertisement."""
+        try:
+            await self._populate_accessories_and_characteristics()
+        except (
+            AccessoryDisconnectedError,
+            *BLEAK_EXCEPTIONS,
+            AccessoryNotFoundError,
+        ) as exc:
+            logger.warning("%s: Failed to process config change: %s", self.name, exc)
+
     async def _async_process_disconnected_events(self) -> None:
+        """Handle disconnected events seen from the advertisement."""
         logger.debug(
             "%s: Polling subscriptions for changes during disconnection", self.name
         )
-        results = await self.get_characteristics(list(self.subscriptions))
+        try:
+            results = await self.get_characteristics(list(self.subscriptions))
+        except (
+            AccessoryDisconnectedError,
+            *BLEAK_EXCEPTIONS,
+            AccessoryNotFoundError,
+        ) as exc:
+            logger.warning(
+                "%s: Failed to fetch disconnected events: %s", self.name, exc
+            )
+            return
+
         for listener in self.listeners:
             listener(results)
 
@@ -360,7 +425,11 @@ class BlePairing(AbstractPairing):
                     tid,
                     iid,
                 ):
-                    await self.client.write_gatt_char(char.handle, data)
+                    await self.client.write_gatt_char(
+                        char.handle,
+                        data,
+                        "write-without-response" not in char.properties,
+                    )
 
                 payload = await self.client.read_gatt_char(char.handle)
 
@@ -405,11 +474,11 @@ class BlePairing(AbstractPairing):
     @retry_bluetooth_connection_error()
     async def list_accessories_and_characteristics(self) -> list[dict[str, Any]]:
         await self._populate_accessories_and_characteristics()
-        return self._accessories.serialize()
+        return self.accessories.serialize()
 
     async def _populate_char_values(self, config_changed: bool) -> None:
         """Populate the values of all characteristics."""
-        for service in self._accessories.aid(1).services:
+        for service in self.accessories.aid(1).services:
             if service.type in SKIP_SYNC_SERVICES:
                 continue
             if (
@@ -461,25 +530,24 @@ class BlePairing(AbstractPairing):
                 # and we are not forcing an update
                 return
 
-            if not self._accessories:
+            if not self.accessories:
                 self._load_accessories_from_cache()
 
             update_values = force_update
             config_changed = False
             if self.description:
-                config_changed = self._config_num != self.description.config_num
+                config_changed = self.config_num != self.description.config_num
 
-            if not self._accessories or config_changed:
+            if not self.accessories or config_changed:
                 logger.debug(
                     "%s: Fetching gatt database because, cached_config_num: %s, adv config_num: %s",
                     self.name,
-                    self._config_num,
+                    self.config_num,
                     self.description.config_num,
                 )
                 accessories = await self._async_fetch_gatt_database()
-                self._accessories_state = AccessoriesState(
-                    accessories, self.description.config_num
-                )
+                new_config_num = self.description.config_num if self.description else 0
+                self._accessories_state = AccessoriesState(accessories, new_config_num)
                 update_values = True
 
             if not self._encryption_key:
@@ -490,13 +558,7 @@ class BlePairing(AbstractPairing):
                 self._update_accessories_state_cache()
 
             if config_changed:
-                self._callback_and_save_config_changed(self._config_num)
-
-            # Only start active subscriptions if we stay connected for more
-            # than subscription delay seconds.
-            self._restore_subscriptions_timer = asyncio.get_event_loop().call_later(
-                SUBSCRIPTION_RESTORE_DELAY, self._restore_subscriptions
-            )
+                self._callback_and_save_config_changed(self.config_num)
 
     def _restore_subscriptions(self):
         """Restore subscriptions after after connecting."""
@@ -509,9 +571,25 @@ class BlePairing(AbstractPairing):
         self, subscriptions: list[tuple[int, int]]
     ) -> None:
         """Start notifications for the given subscriptions."""
+        if not self.accessories or not self.client.is_connected:
+            return
+
         for _, iid in subscriptions:
-            if iid not in self._notifications:
-                await self._async_start_notify(iid)
+            if iid in self._notifications:
+                continue
+            # The iid will not be in in self._notifications until
+            # the _async_start_notify call returns.
+            async with self._subscription_lock:
+                if iid in self._notifications or not self.client.is_connected:
+                    continue
+                try:
+                    await self._async_start_notify(iid)
+                except BLEAK_EXCEPTIONS as ex:
+                    # Likely disconnected before we could start notifications
+                    # we will get disconnected events instead.
+                    logger.debug(
+                        "%s: Could not start notify for %s: %s", self.name, iid, ex
+                    )
 
     @operation_lock
     @retry_bluetooth_connection_error()
@@ -535,7 +613,7 @@ class BlePairing(AbstractPairing):
             ]
         )
 
-        info = self._accessories.aid(1).services.first(
+        info = self.accessories.aid(1).services.first(
             service_type=ServicesTypes.PAIRING
         )
         char = info[CharacteristicsTypes.PAIRING_PAIRINGS]
@@ -569,7 +647,7 @@ class BlePairing(AbstractPairing):
         self,
         characteristics: list[tuple[int, int]],
     ) -> dict[tuple[int, int], dict[str, Any]]:
-        await self._get_characteristics_without_retry(characteristics)
+        return await self._get_characteristics_without_retry(characteristics)
 
     @operation_lock
     async def _get_characteristics_without_retry(
@@ -591,7 +669,7 @@ class BlePairing(AbstractPairing):
             data = await self._async_request(OpCode.CHAR_READ, iid)
             decoded = dict(TLV.decode_bytes(data))[1]
 
-            char = self._accessories.aid(1).characteristics.iid(iid)
+            char = self.accessories.aid(1).characteristics.iid(iid)
             logger.debug(
                 "%s: Read characteristic got data, expected format is %s: data=%s decoded=%s",
                 self.name,
@@ -623,7 +701,7 @@ class BlePairing(AbstractPairing):
         results: dict[tuple[int, int], Any] = {}
 
         for aid, iid, value in characteristics:
-            char = self._accessories.aid(1).characteristics.iid(iid)
+            char = self.accessories.aid(1).characteristics.iid(iid)
 
             if CharacteristicPermissions.timed_write in char.perms:
                 payload_inner = TLV.encode_list(
@@ -656,7 +734,9 @@ class BlePairing(AbstractPairing):
     @operation_lock
     async def subscribe(self, characteristics):
         new_chars = await super().subscribe(characteristics)
-        if not new_chars:
+        if not new_chars or not self.client or not self.client.is_connected:
+            # Don't force a new connection if we are not already
+            # connected as we will get disconnected events.
             return
         logger.debug("%s: subscribing to %s", self.name, new_chars)
         await self._populate_accessories_and_characteristics()
@@ -670,7 +750,7 @@ class BlePairing(AbstractPairing):
     async def identify(self):
         await self._populate_accessories_and_characteristics()
 
-        info = self._accessories.aid(1).services.first(
+        info = self.accessories.aid(1).services.first(
             service_type=ServicesTypes.ACCESSORY_INFORMATION
         )
         char = info[CharacteristicsTypes.IDENTIFY]
@@ -714,7 +794,7 @@ class BlePairing(AbstractPairing):
             ]
         )
 
-        info = self._accessories.aid(1).services.first(
+        info = self.accessories.aid(1).services.first(
             service_type=ServicesTypes.PAIRING
         )
         char = info[CharacteristicsTypes.PAIRING_PAIRINGS]
@@ -757,7 +837,7 @@ class BlePairing(AbstractPairing):
             ]
         )
 
-        info = self._accessories.aid(1).services.first(
+        info = self.accessories.aid(1).services.first(
             service_type=ServicesTypes.PAIRING
         )
         char = info[CharacteristicsTypes.PAIRING_PAIRINGS]
