@@ -15,13 +15,18 @@
 #
 
 import asyncio
+from datetime import timedelta
 from itertools import groupby
 import json
 import logging
 from operator import itemgetter
 from typing import Any
 
-from aiohomekit.controller.abstract import AbstractPairing
+from aiohomekit.controller.abstract import (
+    AbstractController,
+    AbstractPairing,
+    AbstractPairingData,
+)
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
     AuthenticationError,
@@ -32,7 +37,7 @@ from aiohomekit.exceptions import (
     UnpairedError,
 )
 from aiohomekit.http import HttpContentTypes
-from aiohomekit.model import Accessories, AccessoriesState
+from aiohomekit.model import Accessories, AccessoriesState, Transport
 from aiohomekit.model.characteristics import CharacteristicsTypes
 from aiohomekit.protocol import error_handler
 from aiohomekit.protocol.statuscodes import to_status_code
@@ -67,7 +72,9 @@ class IpPairing(AbstractPairing):
     This represents a paired HomeKit IP accessory.
     """
 
-    def __init__(self, controller, pairing_data):
+    def __init__(
+        self, controller: AbstractController, pairing_data: AbstractPairingData
+    ) -> None:
         """
         Initialize a Pairing by using the data either loaded from file or obtained after calling
         Controller.perform_pairing().
@@ -84,6 +91,21 @@ class IpPairing(AbstractPairing):
     def is_connected(self) -> bool:
         return self.connection.is_connected
 
+    @property
+    def is_available(self) -> bool:
+        """Returns true if the device is currently available."""
+        return self.connection.is_connected
+
+    @property
+    def transport(self) -> Transport:
+        """The transport used for the connection."""
+        return Transport.IP
+
+    @property
+    def poll_interval(self) -> timedelta:
+        """Returns how often the device should be polled."""
+        return timedelta(minutes=1)
+
     def event_received(self, event):
         self._callback_listeners(format_characteristic_list(event))
 
@@ -117,6 +139,9 @@ class IpPairing(AbstractPairing):
                 f"Ensure connection returned but still not connected: {self.connection.host}:{self.connection.port}"
             )
 
+        else:
+            self._callback_availability_changed(True)
+
     async def close(self) -> None:
         """
         Close the pairing's communications. This closes the session.
@@ -145,7 +170,7 @@ class IpPairing(AbstractPairing):
                     characteristic["type"] = normalize_uuid(characteristic["type"])
 
         self._accessories_state = AccessoriesState(
-            Accessories.from_list(accessories), self._config_num or 0
+            Accessories.from_list(accessories), self.config_num or 0
         )
         return accessories
 
@@ -219,7 +244,7 @@ class IpPairing(AbstractPairing):
         """
         await self._ensure_connected()
 
-        if not self._accessories:
+        if not self.accessories:
             await self.list_accessories_and_characteristics()
 
         url = "/characteristics?id=" + ",".join(
@@ -244,7 +269,7 @@ class IpPairing(AbstractPairing):
         """
         await self._ensure_connected()
 
-        if not self._accessories:
+        if not self.accessories:
             await self.list_accessories_and_characteristics()
 
         data = []
@@ -344,7 +369,7 @@ class IpPairing(AbstractPairing):
         This method should try not to fetch all the accessories unless
         we know the config num is out of date or force_update is True
         """
-        if not self._accessories or force_update:
+        if not self.accessories or force_update:
             await self.list_accessories_and_characteristics()
 
     async def _process_config_changed(self, config_num: int) -> None:
@@ -356,7 +381,7 @@ class IpPairing(AbstractPairing):
         self._accessories_state = AccessoriesState(
             self._accessories_state.accessories, config_num
         )
-        self._callback_and_save_config_changed(self._config_num)
+        self._callback_and_save_config_changed(self.config_num)
 
     async def identify(self):
         """
@@ -370,15 +395,15 @@ class IpPairing(AbstractPairing):
         """
         await self._ensure_connected()
 
-        if not self._accessories:
+        if not self.accessories:
             await self.list_accessories_and_characteristics()
 
         # we are looking for a characteristic of the identify type
         identify_type = CharacteristicsTypes.IDENTIFY
 
         # search all accessories, all services and all characteristics
-        logger.debug("Searching for identify characteristic in %s", self._accessories)
-        for accessory in self._accessories:
+        logger.debug("Searching for identify characteristic in %s", self.accessories)
+        for accessory in self.accessories:
             aid = accessory.aid
             for service in accessory.services:
                 for characteristic in service.characteristics:
